/* -----------------------------------------------------------------------------
 *
 *
 *  ____                   _____     _
 * / ___|  __ _ _ __  _ __|__  /  __| | ___
 * \___ \ / _` | '_ \| '_ \ / /  / _` |/ _ \
 *  ___) | (_| | |_) | |_) / /_ | (_| |  __/
 * |____/ \__,_| .__/| .__/____(_)__,_|\___|
 *             |_|   |_|
 *
 *
 *
 * Author: Andreas Gschossmann
 * Email:  andreas.gschossmann@hs-regensburg.de
 *
 * quadrotor_dynamics.hh - Relationship between rotor-speed or alternatively
 * digital value for brushless drivers and resulting thrust of the motor.
 *
 * -----------------------------------------------------------------------------*/
#ifndef _CONSTANTS_HH_ 
#define _CONSTANTS_HH_

//////////////
// Aerodynamik
#define EFFICIENCY_CMR			0.8
#define KT_RR282734_10_45_R     	0.00060934	
#define KM_RR282734_10_45_R 		0.00011386*EFFICIENCY_CMR  // durch Elektrische Leistung ermittelt: TODO: Drehmoment messen und k_M fitten

/////////////////////
// PT-1 element motor
#define K_PT1_RR282734_10_45_R		0.378634164294
#define C_PT1_RR282734_10_45_R  	15.1056160953 
#define T_PT1_RR282734_10_45_R  	0.0	
#define DEAD_TIME_PT1_RR282734_10_45_R 	0.0 

/////////////////////////////
// dimensions quadrotor frame
#define MOMENT_ARM			0.22	// distance between motor axis and center of frame
						// Achtung: muss mit sdf-file abgestimmt werden 

/////////////////////////////
// simulation time
#define SIM_TIME			0.001	// time for between to simulation steps
						// Achtung muss mit sdf-file abgestimmt werden 
//#define KT_RR282734_10_45_R_DIG 0.5
//#define KT_RR282734_10_45_R_DiG 0.5

////////////////////////////////////
// Error types for quadrotor_dynamics
#define NO_ERROR        0
#define GAUSSIAN_NOISE  1
#define SINE_ERROR      2

////////////////////////
// Symmetric motor order
// (Andi)
#define FRONT	     	0
#define RIGHT     	1
#define REAR      	2
#define LEFT      	3

#define MOTOR_1     	0
#define MOTOR_2     	1
#define MOTOR_3      	2
#define MOTOR_4      	3

#define M1     		0
#define M2     		1
#define M3      	2
#define M4      	3

#endif
