/* -----------------------------------------------------------------------------
 * 
 *
 *  ____                   _____     _      
 * / ___|  __ _ _ __  _ __|__  /  __| | ___ 
 * \___ \ / _` | '_ \| '_ \ / /  / _` |/ _ \
 *  ___) | (_| | |_) | |_) / /_ | (_| |  __/
 * |____/ \__,_| .__/| .__/____(_)__,_|\___|
 *             |_|   |_| 
 * 
 *
 * 
 * Author: Andreas Gschossmann
 * Email:  andreas.gschossmann@hs-regensburg.de
 * 
 * gazebo_model_plugin.cc - gazebo-plugin to get access to gazebo-API 
 *
 * ----------------------------------------------------------------------------*/ 

#include <iostream>
#include <math.h>
#include <boost/shared_ptr.hpp>

#include <boost/bind.hpp>
#include <gazebo/gazebo.hh>
#include <gazebo/physics/physics.hh>
#include <gazebo/common/common.hh>
#include <stdio.h>

#include <gazebo/transport/transport.hh>
#include <gazebo/msgs/msgs.hh>

#include <boost/shared_ptr.hpp>
#include <sdf/sdf.hh>
#include <boost/gil/gil_all.hpp>
#include <boost/gil/extension/io/png_dynamic_io.hpp>

#include <gazebo/gui/GuiIface.hh>
#include <gazebo/gui/GuiEvents.hh>

#include "gazebo/math/Vector3.hh"
#include "gazebo/physics/physics.hh"
//#include "imu.pb.h"

//#include "quadrotor_dynamics.hh"
//#include "quadrotor_dynamics_types.hh"

#include "../sil/sil.hh"

#include "../dependencies/constants.hh"
#include "gazebo_model_plugin.hh" 

using namespace gazebo;

// Here the myQuadRotorDynamics constructor is saved
GazeboModelPlugin::GazeboModelPlugin() : 
     ModelPlugin(), 
     _myQuadRotorDynamics(KT_RR282734_10_45_R, KM_RR282734_10_45_R, MOMENT_ARM, NO_ERROR),
     // TODO: prüfen und mit korrekten Werten füttern 
     _earth_mag_vec3 (0, 0, 1) // Default value of magnetic 
                               // field vector is set here,
                               // considering the heading of
                               // uav is zero. 
     //, _sil()//, angular_vel()
{ 
}

/*GazeboModelPlugin::~GazeboModelPlugin()
{
}*/

void GazeboModelPlugin::Load(physics::ModelPtr _parent, sdf::ElementPtr /*_sdf*/) 
{

  // Store the pointer to the model  
  this->model = _parent;

  motor_1 = _parent->GetLink("motor_1");
  motor_2 = _parent->GetLink("motor_2");
  motor_3 = _parent->GetLink("motor_3"); 
  motor_4 = _parent->GetLink("motor_4");
  motor_front = _parent->GetLink("motor_1");
  motor_right = _parent->GetLink("motor_2");
  motor_rear = _parent->GetLink("motor_3");
  motor_left = _parent->GetLink("motor_4");

  frame_center = _parent->GetLink("my_uav");

  motor_prop_1 = _parent->GetLink("motor_rod_1");
  motor_prop_2 = _parent->GetLink("motor_rod_2");
  motor_prop_3 = _parent->GetLink("motor_rod_3");
  motor_prop_4 = _parent->GetLink("motor_rod_4");
  motor_prop_front = _parent->GetLink("motor_rod_1");
  motor_prop_right = _parent->GetLink("motor_rod_2");
  motor_prop_rear = _parent->GetLink("motor_rod_3");
  motor_prop_left = _parent->GetLink("motor_rod_4");

  _myQuadRotorDynamics.activate_limit_constant(false);
  _myQuadRotorDynamics.activate_limit_transient(false);
  _myQuadRotorDynamics.activate_gyroscopic_torque(false);
 
  _myQuadRotorDynamics.set_k_pt1(K_PT1_RR282734_10_45_R); 
  _myQuadRotorDynamics.set_c_pt1(C_PT1_RR282734_10_45_R);
  _myQuadRotorDynamics.set_c_pt1(T_PT1_RR282734_10_45_R);
  _myQuadRotorDynamics.set_c_pt1(DEAD_TIME_PT1_RR282734_10_45_R);

  /////////////// 
  // init SIL
  _sil.init();

  // Listen to the update event. This event is broadcast every
  // simulation iteration.
  this->updateConnection = event::Events::ConnectWorldUpdateBegin(
      boost::bind(&GazeboModelPlugin::OnUpdate, this, _1));

  node = transport::NodePtr(new transport::Node()); 
  node->Init();
  std::cout << "Subscribing to: " << "~/quadrotor/my_uav/my_imu/imu" << std::endl;
  imuSubscriber = node->Subscribe("~/quadrotor/my_uav/my_imu/imu", &GazeboModelPlugin::callback_imu, this);

}

/////////////////////////////////////////////////
// Function is called everytime an imu-message is received.
void GazeboModelPlugin::callback_imu(ConstIMUPtr &msg)
{

  // Get imu data (acceleration, gyro)
  _sensor_vals.imu.set_acc( msg->linear_acceleration() );
  _sensor_vals.imu.set_gyro( msg->angular_velocity() );

  // Get ideal magnetic field values (sensor noise not supported yet)
  //_sensor_vals.mag.set( frame_center->GetWorldCoGPose().RotateVector(_earth_mag_vec3) );
  
  // Get actual sensor data
  _sensor_vals.imu.roll_actual = frame_center->GetWorldCoGPose().rot.GetRoll();
  _sensor_vals.imu.pitch_actual = frame_center->GetWorldCoGPose().rot.GetPitch();
  _sensor_vals.imu.yaw_actual = frame_center->GetWorldCoGPose().rot.GetYaw();

}

// Called by the world update start event
void GazeboModelPlugin::OnUpdate(const common::UpdateInfo & /*_info*/)
{

  // Get simulator time
  _current_time = this->model->GetWorld()->GetSimTime(); // simulatin time when this state was generated
  _dt = _current_time - _last_time;
  _last_time = _current_time;
  //std::cout << _dt.Double() << std::endl;
 
  ////////////////// 
  // Update SIL
  _sil.update( _dt.Double(), _sensor_vals); // in dieser Funktion sollte der TestCode stehen
  // Update QuadrotorDynamics
  _mechanics = _myQuadRotorDynamics.update( _sil.get_actuation_value(), _dt.Double() ); // nun werden die Drehzahlwerte in Kräfte und Momente gewandelt

/*
  std::cout << "Real:   "
            << ( frame_center->GetWorldCoGPose().rot.GetPitch() * 180 / M_PI ) << "/"
            << ( frame_center->GetWorldCoGPose().rot.GetRoll()  * 180 / M_PI ) << std::endl;
*/
  
  //////////////////////////////////
  // May the mass times acceleration
  // be with you.
  frame_center->AddRelativeForce( _myQuadRotorDynamics.getThrust().all_vec3() ); 
  frame_center->AddRelativeTorque( _myQuadRotorDynamics.getTorque().vec3() ); //- frame_center->GetInertial()->GetCoG().Cross( _myQuadRotorDynamics.getThrust().all_vec3() ) );

/* 
  // alternative Lösung
  // Set Thrusts 
  motor_front->AddRelativeForce ( _myQuadRotorDynamics.getThrust().vec3(FRONT) );
  motor_rear->AddRelativeForce  ( _myQuadRotorDynamics.getThrust().vec3(REAR)  );
  motor_left->AddRelativeForce  ( _myQuadRotorDynamics.getThrust().vec3(LEFT)  ); 
  motor_right->AddRelativeForce ( _myQuadRotorDynamics.getThrust().vec3(RIGHT) );
  // Set Yaw Torque
  frame_center->AddRelativeTorque ( math::Vector3(0.0, 0.0, _myQuadRotorDynamics.getTorque().yaw) );
*/

}

void GazeboModelPlugin::get_magnetic_field()
{
  std::cout << "Mangetic field times quaternion." << std::endl;
 
  //frame_center->GetWorldCoGPose().RotateVector(_earth_mag_vec3);
}
GZ_REGISTER_MODEL_PLUGIN(GazeboModelPlugin)
