/* -----------------------------------------------------------------------------
 *
 *
 *  ____                   _____     _
 * / ___|  __ _ _ __  _ __|__  /  __| | ___
 * \___ \ / _` | '_ \| '_ \ / /  / _` |/ _ \
 *  ___) | (_| | |_) | |_) / /_ | (_| |  __/
 * |____/ \__,_| .__/| .__/____(_)__,_|\___|
 *             |_|   |_|
 *
 *
 *
 * Author: Andreas Gschossmann
 * Email:  andreas.gschossmann@hs-regensburg.de
 *
 * sil.cc - This class contains the software algorithms which control
 * the quadrocopter and should be tested. 
 *
 * -----------------------------------------------------------------------------*/
#include "sil.hh"
#include "constants.hh"
#include "ps3control.hh"

extern "C" {
  #include "sensor_fusion.h"
  #include "pid.h"
  #include "backstepping.h"
  #include "mixer.h"

  #include "math.h"
  #include "mainvars.h"
}

//using namespace std;

SIL::SIL() : _test_sequence_iterator(0), _ps3controller("/dev/input/js0") // TODO: Joystick Aufruf parametriserbar machen
{         
}

SIL::~SIL()
{
}

/// \brief Get current rotor speed
AngularVel SIL::get_rotor_speed()
{
  return _rotor_speed;
}

/// \brief Get current actuation
ActuationVal SIL::get_actuation_value()
{
  return _actuation_value;
}

void SIL::get_ps3controller() {
  /////////////////////
  // Todo: Alles nur dann ausführen, wenn Controller gefunden wurde
  //
  /*
  if (!_ps3controller.isFound())
  {
    printf("open failed.\n");
    exit(1);
  }*/

  // Attempt to sample an event from the joystick
  JoystickEvent event;
  if (_ps3controller.sample(&event))
  {
    if (event.isButton())
    {
      /*
      if (event.number == 9)
      {
        yaw_ctrl = 2;
      }*/  
      /*
      printf("Button %u is %s\n",
        event.number,
        event.value == 0 ? "up" : "down");*/
    }
    else if (event.isAxis())
    {
      if (event.number == 3)
      {
        //printf("Axis %u is at position %d\n", event.number, event.value);
        gas_ctrl = (-event.value) * PS3_GAIN_GAS;
      }
      if (event.number == 1)
      {
        pitch_ctrl = (event.value) * PS3_GAIN_PITCH;
      }
      if (event.number == 0)
      {
        roll_ctrl = (-event.value) * PS3_GAIN_ROLL;
      }
      if (event.number == 2)
      {
        // läuft noch unstabil und wird deshalb vorerst nicht unterstützt
        yaw_ctrl = 0.0;//(event.value) * PS3_GAIN_YAW;
      }
      /*
      if (event.number == 0 || event.number == 1 || event.number == 2 || event.number == 3)
      {
        //printf("Axis %u is at position %d\n", event.number, event.value);
        gas = event.value/3000;
        std::cout
      }*/
    }
  }

}

void SIL::get_key()
{
  // Noch keinen Schimmer wie das gehen soll, 
  // vlt. über qt?
}

void SIL::test_sequence()
{
  _test_sequence_iterator++;
  
  std::cout << "Test sequence" << std::endl; 

  
  // TODO: hier nur noch actuation values setzen

  if (_test_sequence_iterator < 100)
  {
    gas_ctrl = 159;
  }
  else if (_test_sequence_iterator < 1000)
  {
    gas_ctrl = 159;
    roll_ctrl = 10; 
  }
  else if (_test_sequence_iterator < 1500)
  {
    gas_ctrl = 159;
    roll_ctrl = 0;
  }

}

void SIL::init()
{
  ////////////////////////
  // Ab hier C-Funktionen
  // vor der main loop
  pidInit(&my_PID_roll, 0, PID_ROLL_KP, PID_ROLL_KI, PID_ROLL_KD, SIM_TIME); // dt = const?
  pidInit(&my_PID_pitch, 0, PID_ROLL_KP, PID_ROLL_KI, PID_ROLL_KD, SIM_TIME); // dt = const? 
  //pidInit(&my_PID_yaw, 0, PID_YAW_KP, PID_YAW_KI, PID_YAW_KD, SIM_TIME); // dt = const?

  backsteppingInit(&my_BACKSTEPPING_roll, 0, 0.22, 0.01788375, 30.89, 3.11); // TODO: Check inertia
  backsteppingInit(&my_BACKSTEPPING_pitch, 0, 0.22, 0.01788375, 30.89, 3.11); // TODO: Check inertia
  //backsteppingInit(&my_BACKSTEPPING_yaw, 0, 0.0357675, x, x, SIM_TIME); //TODO: Check inertia

}

void SIL::update(double dT, Sensors mySensors)
{
  // Dies entspricht der Hauptschleife, hier müssen die zu testenden C-Funktionen eingesetzt werden

  ////////////////////////
  // Ab hier C-Funktionen
  // der main loop

  //
  // Set Stimuli 
  // 

  std::cout << "dT: " << dT << std::endl;

  if (_ps3controller.isFound())
  {
    // obtain stimulus from PS3-Controller
    get_ps3controller();
  }
  else
  {
    // no joystick detected, thus custom stimulus
    test_sequence();
  }
//  std::cout << "gas_ctrl: " << gas_ctrl << std::endl;
//  std::cout << "pitch_ctrl: " << pitch_ctrl << std::endl;
//  std::cout << "roll_ctrl: " << roll_ctrl << std::endl;
  //std::cout << "yaw_ctrl: " << yaw_ctrl << std::endl;

  //
  // Set new desired values
  //

//  pidSetDesired(&my_PID_roll, roll_ctrl);
//  pidSetDesired(&my_PID_pitch, pitch_ctrl);
  //pidSetDesired(&my_PID_yaw, yaw_ctrl);

  backsteppingSetDesired(&my_BACKSTEPPING_roll, roll_ctrl);
  backsteppingSetDesired(&my_BACKSTEPPING_pitch, pitch_ctrl); 
  // 
  // obtain sensor data 
  //
 
  // obtain data acceleration
  acc_x = mySensors.imu.acc_x;
  acc_y = mySensors.imu.acc_y;
  acc_z = mySensors.imu.acc_z;
  
  // obtain data gyro 
  gyro_x = mySensors.imu.gyro_x; 
  gyro_z = mySensors.imu.gyro_y;
  gyro_y = mySensors.imu.gyro_z;

  //
  // Compute inertia angles by filter algorithm
  //

  SYS_TIMING_MS = dT * 1000;
  MahonyAHRSupdateIMU((float)gyro_x, (float)gyro_y, (float)gyro_z, (float)acc_x, (float)acc_y, (float)acc_z);
//  std::cout << "acc_xyz: " << acc_x << " " << acc_y << " " << acc_z << std::endl;
  ////////////////////////
  // for Test purposes set angle vals here
//  std::cout << "global_roll:  " << global_roll << std::endl;
//  std::cout << "global_pitch: " << global_pitch << std::endl;

  global_roll = mySensors.imu.roll_actual * 180 / M_PI;
  global_pitch = mySensors.imu.pitch_actual * 180 / M_PI;
  global_yaw = mySensors.imu.yaw_actual * 180 / M_PI;
 
  //
  // Update PID control
  //

//  y_roll = pidUpdate(&my_PID_roll, global_roll, true);
//  y_pitch = pidUpdate(&my_PID_pitch, global_pitch, true);
  //y_yaw = pidUpdate(&my_PID_yaw, global_yaw, true); // not yet

  y_roll = backsteppingUpdate(&my_BACKSTEPPING_roll, global_roll, gyro_x, true); //TODO: gyro_x rad/s oder grad/s
  y_pitch = backsteppingUpdate(&my_BACKSTEPPING_pitch, global_pitch, gyro_y, true);
  //y_yaw = backsteppingUpdate(&my_BACKSTEPPING_yaw, global_yaw, gyro_z, true);
  y_yaw = yaw_ctrl;

 // std::cout << "error_roll:    " << my_PID_roll.error << std::endl;
  //std::cout << "error_pitch:   " << my_PID_pitch.error << std::endl;
  //std::cout << "y_roll:     " << pidUpdate(&my_PID_roll, global_roll, true)   << std::endl;
  //std::cout << "y_pitch:    " << pidUpdate(&my_PID_pitch, global_pitch, true) << std::endl;
  //std::cout << "y_roll_bs:  " << backsteppingUpdate(&my_BACKSTEPPING_roll, global_roll, gyro_x, true) << std::endl;
  //std::cout << "y_pitch_bs: " << backsteppingUpdate(&my_BACKSTEPPING_pitch, global_pitch, gyro_y, true) << std::endl;
  //std::cout << "gas_ctrl: " << y_yaw   << std::endl;

  std::cout << "y_ctrl: " << pitch_ctrl << std::endl;
  
  //
  // Mixer
  //  

if (gas_ctrl!=0) {
  _actuation_value.motor[RIGHT] = gas_ctrl + y_yaw - y_roll; 
  _actuation_value.motor[LEFT] = gas_ctrl + y_yaw + y_roll;
  _actuation_value.motor[FRONT] = gas_ctrl - y_yaw - y_pitch;
  _actuation_value.motor[REAR] = gas_ctrl - y_yaw + y_pitch;

  std::cout << "right: " << _actuation_value.motor[RIGHT] << std::endl; 
  std::cout << "left:  " << _actuation_value.motor[LEFT]  << std::endl;
  std::cout << "front: " << _actuation_value.motor[FRONT] << std::endl;
  std::cout << "rear:  " << _actuation_value.motor[REAR]  << std::endl;
}

//  std::cout << "gas_ctrl: " << gas_ctrl << std::endl;
//  std::cout << "pitch_ctrl: " << pitch_ctrl << std::endl; //<< "front: " << _actuation_value.motor[FRONT] << "rear: " << _actuation_value.motor[REAR] << std::endl;
//  std::cout << "error pitch: " << my_PID_pitch.error << "error roll: " << my_PID_roll.error << std::endl;
//  std::cout << "roll_ctrl: " << roll_ctrl << std::endl;

  // hier 
  //std::cout << "gx, gy, gz, ax, ay, az" << std::endl;
  //std::cout << gyro_x << ", " << gyro_y << ", " << gyro_z << ", " << acc_x << ", " << acc_y << ", " << acc_z << std::endl;
  //std::cout << "Pitch/Roll" << std::endl;
//std::cout << "Filter: " << global_pitch << "/" << global_roll << std::endl;
//std::cout << "Real:   " << mySensors.imu.pitch_actual << "/" << mySensors.imu.yaw_actual << std::endl;
  //TODO: Realposition auf Richtigkeit verifizieren und Feststellen, ob Richtung korrekt ist.
  //double test = - (atan2(-acc_z,acc_y)) - (3.14f/2.0f);
  //test =  - test * (180.0f/3.14f);

  //std::cout << "roll_raw:  " << test << std::endl;
  //std::cout << "roll:      " << roll << " pitch: " << pitch << std::endl;


  //get_joystick(); //TODO: get_joystick vernuenftig einbinden

}
