#ifndef BACKSTEPPING_H_ 
#define BACKSTEPPING_H_

#include <stdbool.h>

typedef struct
{
  float desired;      //< set point
  float error;        //< error
  float prevError;    //< previous error
  float a1;	      //< constant a1
  float a2;	      //< constant a2
  float J;	      //< Inertia (x, y or z)
  float l;            //< length between motor axis and center
  float dt;           //< delta-time dt
} BackSteppingObject;

/**
 * PID object initialization.
 *
 * @param[out] backstepping   A pointer to the backstepping object to initialize.
 */
void backsteppingInit(BackSteppingObject* backstepping, const float desired, const float l, const float J, const float a1, const float a2);

/**
 * Set the integral limit for this PID in deg.
 *
 * @param[in] backstepping   A pointer to the backstepping object.
 */
void backsteppingReset(BackSteppingObject* backstepping);

float backsteppingUpdate(BackSteppingObject* backstepping, const float measured, const float omega, const bool updateError);

void backsteppingSetDesired(BackSteppingObject* backstepping, const float desired);

float backsteppginGetDesired(BackSteppingObject* backstepping);

/**
 * Set a new error. Use if a special error calculation is needed.
 *
 * @param[in] pid   A pointer to the pid object.
 * @param[in] error The new error
 */
void backsteppingSetError(BackSteppingObject* backstepping, const float error);

#endif /* BACKSTEPPING_H_ */
